<?php

namespace Drupal\commerce_cmi\Controller;

use Drupal\commerce_cmi\CmiService;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Controller for handling CMI payment callbacks.
 */
class CmiCallbackController extends ControllerBase {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The logger factory.
   *
   * @var \Drupal\Core\Logger\LoggerChannelFactoryInterface
   */
  protected $loggerFactory;

  /**
   * The CMI service.
   *
   * @var \Drupal\commerce_cmi\CmiService
   */
  protected $cmiService;

  /**
   * Constructs a new CmiCallbackController.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   The logger factory.
   * @param \Drupal\commerce_cmi\CmiService $cmi_service
   *   The CMI service.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, LoggerChannelFactoryInterface $logger_factory, CmiService $cmi_service) {
    $this->entityTypeManager = $entity_type_manager;
    $this->loggerFactory = $logger_factory;
    $this->cmiService = $cmi_service;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager'),
      $container->get('logger.factory'),
      $container->get('commerce_cmi.cmi_service')
    );
  }

  /**
   * Handles the callback request from CMI payment gateway.
   *
   * @param \Symfony\Component\HttpFoundation\Request $request
   *   The HTTP request object.
   *
   * @return \Symfony\Component\HttpFoundation\Response
   *   The HTTP response object.
   */
  public function handleCallback(Request $request) {
    $post_data = $request->request->all();

    $this->loggerFactory->get('commerce_cmi')
      ->notice('Received CMI callback: @data', ['@data' => print_r($post_data, TRUE)]);

    if (empty($post_data['oid']) || empty($post_data['HASH'])) {
      return new Response('Invalid request: Missing required fields', 400);
    }

    try {
      $order = $this->entityTypeManager->getStorage('commerce_order')
        ->load($post_data['oid']);
      if (!$order) {
        return new Response('Order not found', 404);
      }

      $payment = $this->entityTypeManager->getStorage('commerce_payment')
        ->load($post_data['oid']);
      if (!$payment) {
        return new Response('Payment not found', 404);
      }

      $payment_gateway = $payment->getPaymentGateway();
      if (!$payment_gateway) {
        return new Response('Payment gateway not found', 404);
      }

      $configuration = $payment_gateway->getPluginConfiguration();

      if (!$this->cmiService->validateHash($post_data, $configuration['storekey'])) {
        return new Response('Invalid hash', 400);
      }

      if ($post_data['ProcReturnCode'] == '00') {
        $payment->setState('completed');
        $payment->save();
        $order->setState('completed');
        $order->save();
        $this->loggerFactory->get('commerce_cmi')
          ->notice('Payment completed for order @order_id', ['@order_id' => $order->id()]);
        return new Response('ACTION=POSTAUTH');
      }
      else {
        $payment->setState('failed');
        $payment->save();
        $order->setState('payment_failed');
        $order->save();
        $this->loggerFactory->get('commerce_cmi')
          ->warning('Payment failed for order @order_id. Error code: @error_code', [
            '@order_id' => $order->id(),
            '@error_code' => $post_data['ProcReturnCode'],
          ]);
        return new Response('APPROVED');
      }
    }
    catch (\Exception $e) {
      $this->loggerFactory->get('commerce_cmi')
        ->error('Error processing CMI callback: @error', ['@error' => $e->getMessage()]);
      return new Response('Internal server error', 500);
    }
  }

}
